#!/bin/python
# -*- coding: utf-8 -*-
from typing import Union
from flaretool.basemodels import BaseDataModel


class IpInfo(BaseDataModel):
    ipaddr: Union[str, None] = None
    hostname: Union[str, None] = None
    country: Union[str, None] = None


class PunyDomainInfo(BaseDataModel):
    originalvalue: str
    encodevalue: str
    decodevalue: str



#!/bin/python
# -*- coding: utf-8 -*-
from pydantic import BaseModel


class BaseDataModel(BaseModel):
    pass

    def __trace__(self):
        return "\n".join(
            [f"{key}='{val}'" for key, val in self.__dict__.items()
             if not key.startswith("_")]
        )

    def __str__(self) -> str:
        columns = ', '.join([
            '{0}={1}'.format(k, repr(self.__dict__[k]))
            for k in self.__dict__.keys() if not k.startswith("_")
        ])

        return '{0}({1})'.format(
            self.__class__.__name__, columns
        )



#!/bin/python
# -*- coding: utf-8 -*-
"""
Class for interacting with a short URL service.

Warning:
    This class may undergo updates and its usage may change in the near future.
"""
import flaretool
from flaretool.decorators import type_check
from flaretool.errors import AuthenticationError
from flaretool.common import requests
from flaretool.constants import BASE_API_URL
from flaretool.shorturl.errors import *
from flaretool.shorturl.models import ShortUrlInfo
import warnings

__all__ = []


class ShortUrlService:
    """
    Class for interacting with a short URL service.
    WebAPI Wrapper class.

    Warning:
        This class may undergo updates and its usage may change in the near future.
    """

    def __init__(self) -> None:
        """Initialize the ShortUrl class.
        WebAPI Wrapper class.

        Warning:
            This class may undergo updates and its usage may change in the near future.
        """
        message = "This class may undergo updates and its usage may change in the near future."
        warnings.warn(message, DeprecationWarning)
        if not flaretool.api_key:
            raise AuthenticationError(
                "No API key provided. You can set your API key in code using 'flaretool.api_key = <API-KEY>', or you can set the environment variable api_key=<API-KEY>). "
            )

    @type_check
    def _send_request(self, method, data: dict = {}, params: dict = {}) -> dict:
        """Send a request to the short URL service.

        Args:
            method (str): HTTP method for the request.
            data (dict): Data to send in the request (default: {}).
            params (dict): Query parameters for the request (default: {}).

        Returns:
            dict: Response data from the short URL service.

        Raises:
            ShortUrlAuthenticationError: If the response code is 401.
            ShortUrlDataUpdateError: If the response code is 409.
            ShortUrlError: If the response code is not 200.
        """
        base_url = f"{BASE_API_URL}/shorturl"
        params["apikey"] = flaretool.api_key
        result = requests.request(
            method,
            base_url,
            params=params,
            data=data,
        ).json()
        response = result["response"]
        if response == 401:
            raise ShortUrlAuthenticationError(**result)
        if response == 409:
            raise ShortUrlDataUpdateError(**result)
        if response != 200:
            raise ShortUrlError(**result)
        return result

    @type_check
    def get_short_url_info_list(self, id: int = None) -> list[ShortUrlInfo]:
        """Get informations about a short URL.

        Args:
            id (int): ID of the short URL (default: None).

        Returns:
            list[ShortUrlInfo]: Information about the short URL.

        Raises:
            ShortUrlAuthenticationError: If the response code is 401.
            ShortUrlDataUpdateError: If the response code is 409.
            ShortUrlError: If the response code is not 200.
        """
        result = self._send_request(
            "get", params={"id": id if id is not None else 0})["data"]
        return [ShortUrlInfo(**data) for data in result] if result else []

    @type_check
    def create_short_url(self, url: str, limited: int = None) -> ShortUrlInfo:
        """Create a new short URL.

        Args:
            url (str): Long URL to be shortened.
            limited (int, optional): link limited(minutes). Defaults to None.

        Returns:
            ShortUrlInfo: Information about the created short URL.

        Raises:
            ShortUrlAuthenticationError: If the response code is 401.
            ShortUrlDataUpdateError: If the response code is 409.
            ShortUrlError: If the response code is not 200.
        """
        data = {"url": url, "limited": limited} if limited is not None else \
            {"url": url}
        return ShortUrlInfo(**self._send_request("post", data=data)["data"][0])

    @type_check
    def update_short_url(self, url_info: ShortUrlInfo) -> ShortUrlInfo:
        """Update a short URL.

        Args:
            url_info (ShortUrlInfo): Updated information about the short URL.

        Returns:
            ShortUrlInfo: Information about the updated short URL.

        Raises:
            ShortUrlAuthenticationError: If the response code is 401.
            ShortUrlDataUpdateError: If the response code is 409.
            ShortUrlError: If the response code is not 200.
        """
        return ShortUrlInfo(**self._send_request("put", data=url_info.model_dump(), params={
            "id": url_info.id})["data"]["after"][0])

    @type_check
    def delete_short_url(self, url_info: ShortUrlInfo) -> ShortUrlInfo:
        """Delete a short URL.

        Args:
            url_info (ShortUrlInfo): Updated information about the short URL.

        Returns:
            ShortUrlInfo: Information about the delete short URL.

        Raises:
            ShortUrlAuthenticationError: If the response code is 401.
            ShortUrlDataUpdateError: If the response code is 409.
            ShortUrlError: If the response code is not 200.
        """
        return ShortUrlInfo(**self._send_request("delete", params={"id": url_info.id})["data"][0])

    @type_check
    def get_qr_code_raw_data(self, url_info: ShortUrlInfo) -> bytes:
        """Get QR Code raw data

        Args:
            url_info (ShortUrlInfo): target information about the short URL.

        Returns:
            bytes: image bytes data.
        """
        return requests.get(url_info.qr_url).content



#!/bin/python
# -*- coding: utf-8 -*-

from typing import Union
from flaretool.basemodels import BaseDataModel


class AmazonInfo(BaseDataModel):
    result: bool
    url: Union[str, None] = None
    title: Union[str, None] = None
    price: Union[str, int, None] = None
    stock: Union[str, None] = None
    distributor: Union[str, None] = None
    sender: Union[str, None] = None
    evaluation: Union[str, None] = None



#!/bin/python
# -*- coding: utf-8 -*-
import warnings

from flaretool.common import requests
import flaretool
from flaretool.ddns.errors import DdnsAuthenticationError, DdnsError
from flaretool.ddns.models import DdnsInfo
from flaretool.decorators import type_check
from flaretool.errors import AuthenticationError
from flaretool.constants import BASE_API_URL

__all__ = []


class DdnsService:
    """
    Service class for Dynamic DNS (DDNS) functionality.

    Warning:
        This class may undergo updates and its usage may change in the near future.
    """

    def __init__(self) -> None:
        """
        Initialize the DDNS service.

        Warning:
            This class may undergo updates and its usage may change in the near future.
        """
        message = "This class may undergo updates and its usage may change in the near future."
        warnings.warn(message, DeprecationWarning)
        if not flaretool.api_key:
            raise AuthenticationError(
                "No API key provided. You can set your API key in code using 'flaretool.api_key = <API-KEY>', or you can set the environment variable api_key=<API-KEY>). "
            )

    def _send_request(self, method: str, data: dict = {}, params: dict = {}) -> dict:
        """
        Send a request to the DDNS service.

        Args:
            method (str): HTTP method for the request.
            data (dict): Data to send in the request (default: {}).
            params (dict): Query parameters for the request (default: {}).

        Returns:
            dict: Response data from the DDNS service.

        Raises:
            DdnsAuthenticationError: If the response code is 401.
            DdnsError: If the response code is not 200.
        """
        base_url = f"{BASE_API_URL}/ddns"
        params["apikey"] = flaretool.api_key
        response = requests.request(
            method,
            base_url,
            params=params,
            data=data,
        )
        result = response.json()
        if response.status_code == 401:
            raise DdnsAuthenticationError(**result)
        if response.status_code != 200:
            raise DdnsError(**result)
        return result

    @type_check
    def update_ddns(self, host: str, ip: str = None) -> DdnsInfo:
        """
        Update the Dynamic DNS (DDNS) for the specified host.

        Args:
            host (str): The host to update the DDNS for.
            ip (str, optional): The IP address to set for the host. If not provided, the current IP will be used.

        Returns:
            DdnsInfo: The updated DDNS information.

        Raises:
                DdnsAuthenticationError: If the response code is 401.
                DdnsError: If the response code is not 200.

        Example:
            >>> service = DdnsService()
            >>> info = service.update_ddns("example", "192.168.0.100")
            >>> print(info)
            DdnsInfo(result=200, status='success', currentIp='192.168.0.99', updateIp='192.168.0.100', domain='example.○○○.○○')
        """
        data = {"host": host}
        if ip is not None:
            data["ip"] = ip
        return DdnsInfo(**self._send_request("post", data=data))



from _typeshed import Incomplete
from collections.abc import Mapping
from typing_extensions import TypeAlias
from requests.models import Response
from requests.sessions import RequestsCookieJar, _Auth, _Cert, _Data, _Files, _HooksInput, _Params, _TextMapping, _Timeout, _Verify

_HeadersMapping: TypeAlias = Mapping[str, str | bytes]


class requests:
    def request(
        method: str,
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...

    def get(
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...

    def post(
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...

    def put(
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...

    def delete(
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...

    def head(
        url: str | bytes,
        *,
        params: _Params | None = ...,
        data: _Data | None = ...,
        headers: _HeadersMapping | None = ...,
        cookies: RequestsCookieJar | _TextMapping | None = ...,
        files: _Files | None = ...,
        auth: _Auth | None = ...,
        timeout: _Timeout | None = ...,
        allow_redirects: bool = ...,
        proxies: _TextMapping | None = ...,
        hooks: _HooksInput | None = ...,
        stream: bool | None = ...,
        verify: _Verify | None = ...,
        cert: _Cert | None = ...,
        json: Incomplete | None = ...,
    ) -> Response: ...



#!/bin/python
# -*- coding: utf-8 -*-
"""
flaretool python module
[Terms of service](https://main.flarebrow.com/terms)
"""
from flaretool.VERSION import VERSION
from flaretool import nettool
from flaretool.settings import get_settings
from flaretool import logger
import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())

settings = get_settings()
api_key: str = settings.api_key


def get_lib_version():
    return VERSION


def get_latest_version() -> str:
    try:
        from flaretool.common import requests
        return requests.get("https://pypi.org/pypi/flaretool/json").json()["info"]["version"]
    except:
        return VERSION


def check_version():
    current_ver = VERSION
    latest_ver = get_latest_version()
    from distutils.version import StrictVersion
    import warnings
    if StrictVersion(current_ver) < StrictVersion(latest_ver):
        warnings.warn(
            f"flaretool a new version has been released. Please update to the latest version as it has been released."
        )


__version__ = get_lib_version()
__all__ = [
    "api_key",
    # "nettool",
    "settings",
    "logger",
    # "errors",
    "get_lib_version",
    "get_latest_version",
    "check_version",
]

check_version()



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.errors import FlareToolError


class JapaneseHolidaysError(FlareToolError):

    def __init__(self, **kwargs):
        self.message = kwargs.get("message", "")
        super().__init__(self.message)



#!/bin/python
# -*- coding: utf-8 -*-
import warnings
from flaretool.decorators import network_required
from flaretool.constants import BASE_API_URL
from flaretool.funcs.models import AmazonInfo


@network_required
def amazon_info(url: str) -> AmazonInfo:
    """
    Fetches Amazon product information using the provided URL.

    Args:
        url (str): The Amazon product URL.

    Returns:
        AmazonInfo: An instance of AmazonInfo containing the product information.

    Warning:
        This feature is unstable. It is considered experimental and subject to potential changes in future versions.
    """
    message = "This feature is unstable. It is considered experimental and subject to potential changes in future versions."
    warnings.warn(message, DeprecationWarning)
    from flaretool.common import requests
    result = requests.post(f"{BASE_API_URL}/amazon", data={"url": url})
    result_json = result.json()
    return AmazonInfo(**result_json)



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.logger import get_logger

logger = get_logger()


class FlareToolError(Exception):
    """Base exception class for FlareTool errors."""

    def __init__(self, message: str = None) -> None:
        """
        Initialize FlareToolError.

        Args:
            message (str): Optional error message (default: None).
        """
        super().__init__(message)
        self.message = message
        logger.error(repr(self))

    def __str__(self) -> str:
        """
        Return a string representation of the exception.

        Returns:
            str: String representation of the exception.
        """
        return self.message

    def __repr__(self) -> str:
        """
        Return a string representation of the exception.

        Returns:
            str: String representation of the exception.
        """
        columns = ', '.join([
            '{0}={1}'.format(k, repr(self.__dict__[k]))
            for k in self.__dict__.keys() if not k.startswith("_")
        ])

        return '<{0}({1})>'.format(
            self.__class__.__name__, columns
        )


class FlareToolNetworkError(FlareToolError):
    def __init__(
        self,
        message=None,
        http_body=None,
        http_status=None,
        json_body=None,
        headers=None,
        code=None,
        **param,
    ):
        super(FlareToolError, self).__init__(message)

        # if http_body and hasattr(http_body, "decode"):
        #     try:
        #         http_body = http_body.decode("utf-8")
        #     except BaseException:
        #         http_body = (
        #             "<Could not decode body as utf-8. "
        #             "Please contact us through our help center at help.FlaresApp.com.>"
        #         )

        self.message = message
        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.code = code
        self.request_id = self.headers.get("request-id", None)

    # def __str__(self):
    #     msg = self._message or "<empty message>"
    #     if self.request_id is not None:
    #         return "Request {0}: {1}".format(self.request_id, msg)
    #     else:
    #         return msg

    # @property
    # def user_message(self):
    #     return self._message

    # def __repr__(self):
    #     return "%s(message=%r, http_status=%r, request_id=%r)" % (
    #         self.__class__.__name__,
    #         self._message,
    #         self.http_status,
    #         self.request_id,
    #     )


class AuthenticationError(FlareToolError):
    pass



#!/bin/python
# -*- coding: utf-8 -*-
import logging
import flaretool
from logging import Logger
import logging.handlers


def setup_logger(
    loglevel: int = logging.INFO,
    console: bool = False,
    file: bool = False,
    file_name: str = 'flaretool.log',
    rotating: bool = False,
    max_bytes: int = 1024000,
    backup_count: int = 5,
    extra: bool = False,
) -> Logger:
    if max_bytes < 0:
        raise ValueError("max_bytes must be a non-negative value")
    if backup_count < 0:
        raise ValueError("backup_count must be a non-negative value")

    logger = logging.getLogger(flaretool.__name__)
    logger.setLevel(loglevel)
    logger.handlers.clear()

    extra_string = ""

    if extra:
        extra_string = "- %(pathname)s:%(lineno)d - %(funcName)s "
    format_string = f"[%(asctime)s] %(levelname)s {extra_string}: %(message)s"
    formatter = logging.Formatter(format_string)

    file_handler = logging.FileHandler(file_name)
    file_handler.setLevel(loglevel)
    file_handler.setFormatter(formatter)

    console_handler = logging.StreamHandler()
    console_handler.setLevel(loglevel)
    console_handler.setFormatter(formatter)

    rotating_handler = logging.handlers.RotatingFileHandler(
        file_name,
        encoding='utf-8',
        maxBytes=max_bytes,
        backupCount=backup_count,
    )
    rotating_handler.setLevel(loglevel)
    rotating_handler.setFormatter(formatter)

    if file:
        logger.addHandler(rotating_handler if rotating else file_handler)

    if console:
        logger.addHandler(console_handler)

    return logger


def get_logger() -> Logger:
    return logging.getLogger(flaretool.__name__)



#!/bin/python
# -*- coding: utf-8 -*-
from datetime import datetime
from flaretool.basemodels import BaseDataModel
from typing import Union


class ShortUrlInfo(BaseDataModel):
    id: int
    url: str
    title: str
    code: str
    disabled: bool
    insert_time: datetime
    limit_time: Union[datetime, str]
    link: str
    qr_url: str

    def __init__(self, *args, **param):
        limit_time = param.pop("limit_time")
        try:
            limit_time = datetime.strptime(limit_time, "%Y-%m-%d %H:%M:%S")
        except:
            pass
        param["limit_time"] = limit_time
        super(BaseDataModel, self).__init__(*args, **param)

    def __sub__(self, other):
        diff = {}
        for field, value in other.model_dump().items():
            if field not in diff and self.model_dump().get(field) != value:
                diff[field] = value
        return diff



#!/bin/python
# -*- coding: utf-8 -*-

from enum import Enum


class ConversionMode(Enum):
    HALF_WIDTH = 1
    FULL_WIDTH = 2
    UPPER = 3
    LOWER = 4


class Base64Mode(Enum):
    ENCODE = 1
    DECODE = 2


class HashMode(Enum):
    MD5 = 1
    SHA1 = 2
    SHA256 = 3
    SHA512 = 4



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.nettool.common import (
    lookup_domain,
    lookup_ip,
    get_global_ipaddr_info,
    domain_exists,
    is_ip_in_allowed_networks,
    get_global_ipaddr_info,
    is_country_ip,
    get_country_ip_list,
    get_japanip_list,
    is_japan_ip,
    get_puny_code,
    get_adhost,
    get_robots_txt_url,
    is_scraping_allowed,
    IpInfo,
    PunyDomainInfo,
)

__all__ = [
    "lookup_domain",
    "lookup_ip",
    "get_global_ipaddr_info",
    "domain_exists",
    "is_ip_in_allowed_networks",
    "get_global_ipaddr_info",
    "get_japanip_list",
    "is_country_ip",
    "get_country_ip_list",
    "is_japan_ip",
    "get_puny_code",
    "get_adhost",
    "get_robots_txt_url",
    "is_scraping_allowed",
    # "IpInfo",
    # "PunyDomainInfo",
]



#!/bin/python
# -*- coding: utf-8 -*-
from datetime import datetime, date
from flaretool.basemodels import BaseDataModel
from pydantic import Field
from typing import Union


class supported(BaseDataModel):
    from_date: date = Field(alias="from")
    to_date: date = Field(alias="to")


class HolidaysResponseModel(BaseDataModel):
    title: str
    author: str
    version: str
    status: bool
    message: str
    updated: datetime
    supported: supported
    count: int
    holidays: Union[dict[date, str], None]



#!/bin/python
# -*- coding: utf-8 -*-
from enum import Enum

BASE_API_URL = "https://api.flarebrow.com/v2"

ADHOST_DATA_URL = "https://raw.githubusercontent.com/flarebrow/public/master/Adhost/hostlist.txt"

COUNTRY_IPV4_DATA_URL = "https://raw.githubusercontent.com/flarebrow/public/master/IPAddress/country/{cc}.txt"


class ConsoleColor:
    BLACK = '\033[30m'  # (文字)黒
    RED = '\033[31m'  # (文字)赤
    GREEN = '\033[32m'  # (文字)緑
    YELLOW = '\033[33m'  # (文字)黄
    BLUE = '\033[34m'  # (文字)青
    MAGENTA = '\033[35m'  # (文字)マゼンタ
    CYAN = '\033[36m'  # (文字)シアン
    WHITE = '\033[37m'  # (文字)白
    COLOR_DEFAULT = '\033[39m'  # 文字色をデフォルトに戻す
    BOLD = '\033[1m'  # 太字
    UNDERLINE = '\033[4m'  # 下線
    INVISIBLE = '\033[08m'  # 不可視
    REVERCE = '\033[07m'  # 文字色と背景色を反転
    BG_BLACK = '\033[40m'  # (背景)黒
    BG_RED = '\033[41m'  # (背景)赤
    BG_GREEN = '\033[42m'  # (背景)緑
    BG_YELLOW = '\033[43m'  # (背景)黄
    BG_BLUE = '\033[44m'  # (背景)青
    BG_MAGENTA = '\033[45m'  # (背景)マゼンタ
    BG_CYAN = '\033[46m'  # (背景)シアン
    BG_WHITE = '\033[47m'  # (背景)白
    BG_DEFAULT = '\033[49m'  # 背景色をデフォルトに戻す
    RESET = '\033[0m'  # 全てリセット


class Country(Enum):
    AD = "Andorra"
    AE = "United Arab Emirates"
    AF = "Afghanistan"
    AG = "Antigua and Barbuda"
    AI = "Anguilla"
    AL = "Albania"
    AM = "Armenia"
    AO = "Angola"
    AQ = "Antarctica"
    AR = "Argentina"
    AS = "American Samoa"
    AT = "Austria"
    AU = "Australia"
    AW = "Aruba"
    AX = "Åland Islands"
    AZ = "Azerbaijan"
    BA = "Bosnia and Herzegovina"
    BB = "Barbados"
    BD = "Bangladesh"
    BE = "Belgium"
    BF = "Burkina Faso"
    BG = "Bulgaria"
    BH = "Bahrain"
    BI = "Burundi"
    BJ = "Benin"
    BL = "Saint Barthélemy"
    BM = "Bermuda"
    BN = "Brunei Darussalam"
    BO = "Bolivia, Plurinational State of"
    BQ = "Bonaire, Sint Eustatius and Saba"
    BR = "Brazil"
    BS = "Bahamas"
    BT = "Bhutan"
    BW = "Botswana"
    BY = "Belarus"
    BZ = "Belize"
    CA = "Canada"
    CD = "Congo, The Democratic Republic of the"
    CF = "Central African Republic"
    CG = "Congo"
    CH = "Switzerland"
    CI = "Côte d'Ivoire"
    CK = "Cook Islands"
    CL = "Chile"
    CM = "Cameroon"
    CN = "China"
    CO = "Colombia"
    CR = "Costa Rica"
    CU = "Cuba"
    CV = "Cabo Verde"
    CW = "Curaçao"
    CY = "Cyprus"
    CZ = "Czechia"
    DE = "Germany"
    DJ = "Djibouti"
    DK = "Denmark"
    DM = "Dominica"
    DO = "Dominican Republic"
    DZ = "Algeria"
    EC = "Ecuador"
    EE = "Estonia"
    EG = "Egypt"
    ER = "Eritrea"
    ES = "Spain"
    ET = "Ethiopia"
    FI = "Finland"
    FJ = "Fiji"
    FK = "Falkland Islands (Malvinas)"
    FM = "Micronesia, Federated States of"
    FO = "Faroe Islands"
    FR = "France"
    GA = "Gabon"
    GB = "United Kingdom"
    GD = "Grenada"
    GE = "Georgia"
    GF = "French Guiana"
    GG = "Guernsey"
    GH = "Ghana"
    GI = "Gibraltar"
    GL = "Greenland"
    GM = "Gambia"
    GN = "Guinea"
    GP = "Guadeloupe"
    GQ = "Equatorial Guinea"
    GR = "Greece"
    GT = "Guatemala"
    GU = "Guam"
    GW = "Guinea-Bissau"
    GY = "Guyana"
    HK = "Hong Kong"
    HN = "Honduras"
    HR = "Croatia"
    HT = "Haiti"
    HU = "Hungary"
    ID = "Indonesia"
    IE = "Ireland"
    IL = "Israel"
    IM = "Isle of Man"
    IN = "India"
    IO = "British Indian Ocean Territory"
    IQ = "Iraq"
    IR = "Iran, Islamic Republic of"
    IS = "Iceland"
    IT = "Italy"
    JE = "Jersey"
    JM = "Jamaica"
    JO = "Jordan"
    JP = "Japan"
    KE = "Kenya"
    KG = "Kyrgyzstan"
    KH = "Cambodia"
    KI = "Kiribati"
    KM = "Comoros"
    KN = "Saint Kitts and Nevis"
    KP = "Korea, Democratic People's Republic of"
    KR = "Korea, Republic of"
    KW = "Kuwait"
    KY = "Cayman Islands"
    KZ = "Kazakhstan"
    LA = "Lao People's Democratic Republic"
    LB = "Lebanon"
    LC = "Saint Lucia"
    LI = "Liechtenstein"
    LK = "Sri Lanka"
    LR = "Liberia"
    LS = "Lesotho"
    LT = "Lithuania"
    LU = "Luxembourg"
    LV = "Latvia"
    LY = "Libya"
    MA = "Morocco"
    MC = "Monaco"
    MD = "Moldova, Republic of"
    ME = "Montenegro"
    MF = "Saint Martin (French part)"
    MG = "Madagascar"
    MH = "Marshall Islands"
    MK = "North Macedonia"
    ML = "Mali"
    MM = "Myanmar"
    MN = "Mongolia"
    MO = "Macao"
    MP = "Northern Mariana Islands"
    MQ = "Martinique"
    MR = "Mauritania"
    MS = "Montserrat"
    MT = "Malta"
    MU = "Mauritius"
    MV = "Maldives"
    MW = "Malawi"
    MX = "Mexico"
    MY = "Malaysia"
    MZ = "Mozambique"
    NA = "Namibia"
    NC = "New Caledonia"
    NE = "Niger"
    NF = "Norfolk Island"
    NG = "Nigeria"
    NI = "Nicaragua"
    NL = "Netherlands"
    NO = "Norway"
    NP = "Nepal"
    NR = "Nauru"
    NU = "Niue"
    NZ = "New Zealand"
    OM = "Oman"
    PA = "Panama"
    PE = "Peru"
    PF = "French Polynesia"
    PG = "Papua New Guinea"
    PH = "Philippines"
    PK = "Pakistan"
    PL = "Poland"
    PM = "Saint Pierre and Miquelon"
    PR = "Puerto Rico"
    PS = "Palestine, State of"
    PT = "Portugal"
    PW = "Palau"
    PY = "Paraguay"
    QA = "Qatar"
    RE = "Réunion"
    RO = "Romania"
    RS = "Serbia"
    RU = "Russian Federation"
    RW = "Rwanda"
    SA = "Saudi Arabia"
    SB = "Solomon Islands"
    SC = "Seychelles"
    SD = "Sudan"
    SE = "Sweden"
    SG = "Singapore"
    SI = "Slovenia"
    SK = "Slovakia"
    SL = "Sierra Leone"
    SM = "San Marino"
    SN = "Senegal"
    SO = "Somalia"
    SR = "Suriname"
    SS = "South Sudan"
    ST = "Sao Tome and Principe"
    SV = "El Salvador"
    SX = "Sint Maarten (Dutch part)"
    SY = "Syrian Arab Republic"
    SZ = "Eswatini"
    TC = "Turks and Caicos Islands"
    TD = "Chad"
    TG = "Togo"
    TH = "Thailand"
    TJ = "Tajikistan"
    TK = "Tokelau"
    TL = "Timor-Leste"
    TM = "Turkmenistan"
    TN = "Tunisia"
    TO = "Tonga"
    TR = "Turkey"
    TT = "Trinidad and Tobago"
    TV = "Tuvalu"
    TW = "Taiwan, Province of China"
    TZ = "Tanzania, United Republic of"
    UA = "Ukraine"
    UG = "Uganda"
    US = "United States"
    UY = "Uruguay"
    UZ = "Uzbekistan"
    VA = "Holy See (Vatican City State)"
    VC = "Saint Vincent and the Grenadines"
    VE = "Venezuela, Bolivarian Republic of"
    VG = "Virgin Islands, British"
    VI = "Virgin Islands, U.S."
    VN = "Viet Nam"
    VU = "Vanuatu"
    WF = "Wallis and Futuna"
    WS = "Samoa"
    YE = "Yemen"
    YT = "Mayotte"
    ZA = "South Africa"
    ZM = "Zambia"
    ZW = "Zimbabwe"



#!/bin/python
# -*- coding: utf-8 -*-
import os
import sys
import json
import platform
import requests as req
from requests.models import Response
from flaretool.logger import get_logger
import flaretool
from flaretool.errors import FlareToolNetworkError

logger = get_logger()


class requests():

    @staticmethod
    def request(method: str, url: str, **kwargs) -> Response:
        from urllib.parse import urlparse
        is_flare_service = False
        headers = kwargs.pop("headers", {})
        ua = {
            "Mozilla": "5.0",
            "publisher": flaretool.__name__,
            "flaretool": flaretool.__version__,
            "lang_version": platform.python_version(),
            "os": platform.system(),
            "platform": platform.platform(),
        }
        user_agent = " ".join([f"{key}/{value}" for key, value in ua.items()])
        headers["User-Agent"] = user_agent
        headers["X-UA"] = user_agent
        if "flarebrow.com" in urlparse(url).netloc:
            is_flare_service = True
            headers["X-FLAREBROW-AUTH"] = flaretool.api_key
        with req.Session() as session:
            response = session.request(
                method=method, url=url, headers=headers, **kwargs)
            logger.debug(
                {
                    "status_code": response.status_code,
                    "method": method,
                    "url": url,
                    "param": kwargs.get("param", {}),
                    "data": kwargs.get("data", {}),
                }
            )
            if is_flare_service and response.status_code == 403:
                raise FlareToolNetworkError(
                    message="Only access from Japan is accepted"
                )
            return response

    @staticmethod
    def get(url, **kwargs) -> Response:
        return requests.request("GET", url, **kwargs)

    @staticmethod
    def post(url, **kwargs) -> Response:
        return requests.request("POST", url, **kwargs)

    @staticmethod
    def put(url, **kwargs) -> Response:
        return requests.request("PUT", url, **kwargs)

    @staticmethod
    def delete(url, **kwargs) -> Response:
        return requests.request("DELETE", url, **kwargs)

    @staticmethod
    def head(url, **kwargs) -> Response:
        kwargs.setdefault("allow_redirects", False)
        return requests.request("HEAD", url, **kwargs)



#!/bin/python
# -*- coding: utf-8 -*-
import datetime
import calendar
from typing import Union

__all__ = []


class JapaneseHolidays:
    """
    日本の祝日を管理するクラス(オフライン版)
    """
    _additional_holidays: dict[datetime.date, str]

    def __init__(self):
        self._additional_holidays = {}

    def is_new_year(self, date: datetime.date) -> bool:
        """
        元日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 元日であればTrue、そうでなければFalse
        """
        return date.month == 1 and date.day == 1

    def is_coming_of_age_day(self, date: datetime.date) -> bool:
        """
        成人の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 成人の日であればTrue、そうでなければFalse
        """
        if date.year < 2000:
            return date.month == 1 and date.day == 15
        return date.month == 1 and date.weekday() == 0 and 8 <= date.day <= 14

    def is_foundation_day(self, date: datetime.date) -> bool:
        """
        建国記念日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 建国記念日であればTrue、そうでなければFalse
        """
        if date.year < 1967:
            return False
        return date.month == 2 and date.day == 11

    def is_spring_equinox(self, date: datetime.date) -> bool:
        """
        春分の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 春分の日であればTrue、そうでなければFalse
        """
        if date.year < 1900 or date.year > 2099:
            return False
        spring_equinox = 20.8431 + 0.242194 * \
            (date.year - 1980) - (date.year - 1980) // 4
        return date == datetime.date(date.year, 3, int(spring_equinox))

    def is_showa_day(self, date: datetime.date) -> bool:
        """
        昭和の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 昭和の日であればTrue、そうでなければFalse
        """
        if 1989 <= date.year <= 2006:
            return False
        return date.month == 4 and date.day == 29

    def is_constitution_day(self, date: datetime.date) -> bool:
        """
        憲法記念日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 憲法記念日であればTrue、そうでなければFalse
        """
        return date.month == 5 and date.day == 3

    def is_greenery_day(self, date: datetime.date) -> bool:
        """
        みどりの日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: みどりの日であればTrue、そうでなければFalse
        """
        if date.year < 1989:
            return False
        if 1989 <= date.year <= 2006:
            return date.month == 4 and date.day == 29
        return date.month == 5 and date.day == 4

    def is_childrens_day(self, date: datetime.date) -> bool:
        """
        こどもの日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: こどもの日であればTrue、そうでなければFalse
        """
        return date.month == 5 and date.day == 5

    def is_marine_day(self, date: datetime.date) -> bool:
        """
        海の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 海の日であればTrue、そうでなければFalse
        """
        if date.year < 1996:
            return False
        if date.month != 7:
            return False
        if date.year < 2003:
            return date.day == 20
        if date.year == 2020:
            return date.day == 23
        if date.year == 2021:
            return date.day == 22
        return date.weekday() == 0 and 15 <= date.day <= 21

    def is_mountain_day(self, date: datetime.date) -> bool:
        """
        山の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 山の日であればTrue、そうでなければFalse
        """
        if date.year < 2016:
            return False
        if date.year == 2020 and date.month == 8:
            return date.day == 10
        if date.year == 2021 and date.month == 8:
            return date.day == 8
        return date.month == 8 and date.day == 11

    def is_respect_for_the_aged_day(self, date: datetime.date) -> bool:
        """
        敬老の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 敬老の日であればTrue、そうでなければFalse
        """
        if date.year <= 2002:
            return date.month == 9 and date.day == 15
        return date.month == 9 and date.weekday() == 0 and 15 <= date.day <= 21

    def is_autumn_equinox(self, date: datetime.date) -> bool:
        """
        秋分の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 秋分の日であればTrue、そうでなければFalse
        """
        if date.year < 1900 or date.year > 2099:
            return False
        autumn_equinox = 23.2488 + 0.242194 * \
            (date.year - 1980) - (date.year - 1980) // 4
        return date == datetime.date(date.year, 9, int(autumn_equinox))

    def is_health_and_sports_day(self, date: datetime.date) -> bool:
        """
        スポーツの日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: スポーツの日であればTrue、そうでなければFalse
        """
        if date.year < 2000:
            return date.month == 10 and date.day == 10
        if date.year == 2020:
            return date == datetime.date(2020, 7, 24)
        if date.year == 2021:
            return date == datetime.date(2021, 7, 23)
        if date.year >= 2020 and date.month == 10 and date.day == self.week_day(date, 2, 1).day:
            return True

        return date.month == 10 and date.weekday() == 0 and 8 <= date.day <= 14

    def is_culture_day(self, date: datetime.date) -> bool:
        """
        文化の日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 文化の日であればTrue、そうでなければFalse
        """
        return date.month == 11 and date.day == 3

    def is_labour_thanksgiving_day(self, date: datetime.date) -> bool:
        """
        勤労感謝の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 勤労感謝であればTrue、そうでなければFalse
        """
        return date.month == 11 and date.day == 23

    def is_emperors_birthday(self, date: datetime.date) -> bool:
        """
        天皇誕生日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 天皇誕生日であればTrue、そうでなければFalse
        """
        # 1948-1988年
        if date.year in range(1948, 1988 + 1) and date.month == 4 and date.day == 29:
            return True
        # 1989-2018年
        # 2019: 国民の祝日に関する法律(昭和23年法律第178号)の一部改正
        elif date.year in range(1989, 2018 + 1) and date.month == 12 and date.day == 23:
            return True
        # 2019: 国民の祝日に関する法律(昭和23年法律第178号)の一部改正
        elif date.year >= 2020 and date.month == 2 and date.day == 23:
            return True
        return False

    def is_national_holiday(self, date: datetime.date) -> bool:
        """
        国民の休日の判定

        Args:
            date (datetime.date): 判定する日付

        Returns:
            bool: 国民の休日であればTrue、そうでなければFalse
        """
        if datetime.date(2019, 4, 30) <= date <= datetime.date(2019, 5, 2):
            return True
        if date in [
            datetime.date(1989, 2, 24),
            datetime.date(1990, 11, 12),
            datetime.date(1993, 6, 9),
            datetime.date(2019, 10, 22),
        ]:
            return True
        if 2007 <= date.year and date.month == 5:
            return False
        if date.year <= 1985:
            return False
        previous_date = date - datetime.timedelta(days=1)
        next_date = date + datetime.timedelta(days=1)
        is_may_holiday = self.is_constitution_day(
            previous_date) and self.is_childrens_day(next_date)
        is_sep_holiday = self.is_respect_for_the_aged_day(
            previous_date) and self.is_autumn_equinox(next_date)
        if (is_may_holiday or is_sep_holiday) and date.weekday() < 6:
            return True
        return False

    def is_transfer_holiday(self, date: datetime.date) -> bool:
        """
        振替休日チェック

        Args:
            date (datetime.date): 日付

        Returns:
            bool: 振替休日の場合はTrue、そうでない場合はFalse
        """
        if date.weekday() == 0:  # 月曜日の場合
            return bool(self.get_holiday_name(date - datetime.timedelta(days=1)))
        if date.year in [2008, 2009, 2014, 2015, 2020] and date.month == 5 and date.day == 6:
            return True
        return False

    def is_additional_holiday(self, date: datetime.date) -> bool:
        """
        追加休日チェック

        Args:
            date (datetime.date): 日付

        Returns:
            bool: 追加休日の場合はTrue、そうでない場合はFalse
        """
        return date in self._additional_holidays.keys()

    def set_additional_holiday(self, name: str, date: datetime.date):
        """
        独自の休日を追加

        Args:
            name (str): 休日名
            date (datetime.date): 休日の日付
        """
        self._additional_holidays[date] = name

    def week_day(self, date: datetime.date, week: int, weekday: int) -> datetime.date:
        """
        指定された日付の週と曜日に該当する日付を取得

        Args:
            date (datetime.date): 日付
            week (int): 週 (1から5)
            weekday (int): 曜日 (1から7, 月曜日を1とする)

        Returns:
            datetime.date: 週と曜日に該当する日付
        """
        if week < 1 or week > 5:
            return None
        if weekday < 1 or weekday > 7:
            return None
        lines = calendar.monthcalendar(date.year, date.month)
        days = []
        for line in lines:
            if line[weekday - 1] == 0:
                continue
            days.append(line[weekday - 1])
        return datetime.date(date.year, date.month, days[week - 1])

    @staticmethod
    def to_date(date: Union[
        str, datetime.datetime, datetime.date
    ]) -> datetime.date:
        """
        日付文字列をdateオブジェクトに変換します

        Args:
            date (str or datetime or date): 変換する日付文字列。

        Returns:
            date: 変換されたdateオブジェクト。

        Raises:
            ValueError: サポートされていない日付形式の場合に発生します。
        """
        if isinstance(date, datetime.datetime):
            return date.date()
        if isinstance(date, datetime.date):
            return date

        formats = [
            '%Y-%m-%d',           # 2021-01-01
            '%Y-%m-%d %H:%M',     # 2021-01-01 12:34
            '%Y-%m-%d %H:%M:%S',  # 2021-01-01 12:34:56
            '%Y/%m/%d',           # 2021/01/01
            '%Y/%m/%d %H:%M',     # 2021/01/01 12:34
            '%Y/%m/%d %H:%M:%S',  # 2021/01/01 12:34:56
            '%Y%m%d',             # 20210101
            '%d-%b-%Y',           # 01-Jan-2021
            '%d-%b-%Y %H:%M:%S',  # 01-Jan-2021 12:34:56
            '%m-%d-%Y',           # 01-01-2021
            '%b %d, %Y',          # Jan 01, 2021
            '%B %d, %Y',          # January 01, 2021
            '%d %b %Y',           # 01 Jan 2021
            '%d %B %Y',           # 01 January 2021
        ]

        for fmt in formats:
            try:
                return datetime.datetime.strptime(date, fmt).date()
            except ValueError:
                pass

        raise ValueError("Unsupported date format")

    @staticmethod
    def get_last_day(date: datetime.date) -> datetime.date:
        """
        指定された月の最終日を取得

        Args:
            date (datetime.date): 月を指定した日付

        Returns:
            datetime.date: 最終日の日付オブジェクト
        """
        last_day = calendar.monthrange(date.year, date.month)[1]
        last_date = datetime.date(date.year, date.month, last_day)
        return last_date

    def get_holiday_name(self, date: Union[
        str, datetime.datetime, datetime.date
    ]) -> Union[str, None]:
        """
        祝日名を取得

        Args:
            date (datetime or date or str): 日時

        Returns:
            str: 祝日名（祝日ではない場合はNone）
        """
        date = self.to_date(date)
        if self.is_new_year(date):
            return "元日"
        elif self.is_coming_of_age_day(date):
            return "成人の日"
        elif self.is_foundation_day(date):
            return "建国記念の日"
        elif self.is_spring_equinox(date):
            return "春分の日"
        elif self.is_showa_day(date):
            if date.year < 1990:
                return "天皇誕生日"
            return "昭和の日"
        elif self.is_constitution_day(date):
            return "憲法記念日"
        elif self.is_national_holiday(date):
            return "国民の休日"
        elif self.is_greenery_day(date):
            return "みどりの日"
        elif self.is_childrens_day(date):
            return "こどもの日"
        elif self.is_marine_day(date):
            return "海の日"
        elif self.is_mountain_day(date):
            return "山の日"
        elif self.is_respect_for_the_aged_day(date):
            return "敬老の日"
        elif self.is_autumn_equinox(date):
            return "秋分の日"
        elif self.is_health_and_sports_day(date):
            if date.year < 2020:
                return "体育の日"
            return "スポーツの日"
        elif self.is_culture_day(date):
            return "文化の日"
        elif self.is_labour_thanksgiving_day(date):
            return "勤労感謝の日"
        elif self.is_emperors_birthday(date):
            return "天皇誕生日"
        elif self.is_transfer_holiday(date):
            return self.get_holiday_name(date - datetime.timedelta(days=1)) + "（振替休日）"
        elif self.is_additional_holiday(date):
            return self._additional_holidays.get(date)
        else:
            return None

    def get_holidays(self, date: str) -> list[tuple[str, datetime.datetime]]:
        """
        祝日の一覧を取得

        Args:
            date (str): 年 or 年月 の日時

        Raises:
            ValueError: 日時のフォーマットエラー

        Returns:
            list[tuple[str, datetime.datetime]]: 祝日リスト

        Example:
            >>> get_holidays("2023")
            >>> get_holidays("2023-01")
            >>> get_holidays("2023/01")
            >>> get_holidays("202301")
        """
        start_date = date
        try:
            start_date = datetime.datetime.strptime(start_date, '%Y').date()
            end_date = datetime.date(start_date.year, 12, 31)
        except ValueError:
            pass

        if isinstance(start_date, str):
            for fmt in ['%Y/%m', '%Y-%m', '%Y%m']:
                try:
                    start_date = datetime.datetime.strptime(
                        start_date, fmt).date()
                    end_date = self.get_last_day(start_date)
                    break
                except ValueError:
                    pass

        if isinstance(start_date, str):
            for fmt in ['%Y/%m/%d', '%Y-%m-%d', '%Y%m%d']:
                try:
                    start_date = datetime.datetime.strptime(
                        start_date, fmt).date()
                    end_date = start_date
                    break
                except ValueError:
                    pass

        if not isinstance(start_date, datetime.date):
            raise ValueError("Unsupported date format")

        holidays = []
        current_date = start_date
        while current_date <= end_date:
            holiday_name = self.get_holiday_name(current_date)
            if holiday_name is not None:
                holidays.append((holiday_name, current_date))
            current_date += datetime.timedelta(days=1)
        return holidays

    def get_rest_days_in_range(self, start_date: datetime.date, end_date: datetime.date) -> list[tuple[str, datetime.datetime]]:
        """
        特定の期間内の休みの一覧を取得（土日含む）

        Args:
            start_date (datetime.date): 開始日
            end_date (datetime.date): 終了日

        Returns:
            list[tuple[str, datetime.datetime]]: 休み一覧
        """
        holidays = []
        current_date = start_date
        while current_date <= end_date:
            if current_date.weekday() == 5:
                holidays.append(("土曜日", current_date))
            elif current_date.weekday() == 6:
                holidays.append(("日曜日", current_date))
            else:
                holiday_name = self.get_holiday_name(current_date)
                if holiday_name is not None:
                    holidays.append((holiday_name, current_date))
            current_date += datetime.timedelta(days=1)
        return holidays

    def get_holidays_in_range(self, start_date: datetime.date, end_date: datetime.date) -> list[tuple[str, datetime.datetime]]:
        """
        特定の期間内の祝日一覧を取得

        Args:
            start_date (datetime.date): 開始日
            end_date (datetime.date): 終了日

        Returns:
            list[tuple[str, datetime.datetime]]: 祝日一覧
        """
        holidays = []
        current_date = start_date
        while current_date <= end_date:
            holiday_name = self.get_holiday_name(current_date)
            if holiday_name is not None:
                holidays.append((holiday_name, current_date))
            current_date += datetime.timedelta(days=1)
        return holidays

    def get_business_date_range(self, start_date: datetime.date, end_date: datetime.date) -> list[datetime.date]:
        """
        指定期間内の営業日の一覧を取得

        Args:
            start_date (datetime.date): 開始日
            end_date (datetime.date): 終了日

        Returns:
            list[datetime.date]: 日付のリスト
        """
        business_days = []
        current_date = start_date
        while current_date <= end_date:
            if current_date.weekday() < 5:  # 土曜日(5)と日曜日(6)を除外
                holiday_name = self.get_holiday_name(current_date)
                if holiday_name is None:
                    business_days.append(current_date)
            current_date += datetime.timedelta(days=1)
        return business_days

    def get_first_business_day(self, date: datetime.date, days: int = 1) -> datetime.date:
        """
        指定された月の営業日を取得（デフォルトは第1営業日）

        Args:
            date (datetime.date): 日付
            days (int): 営業日数（デフォルトは1）

        Returns:
            datetime.date: 営業日
        """
        return self.get_business_date_range(
            datetime.date(date.year, date.month, 1),
            self.get_last_day(datetime.date(date.year, date.month, 1)),
        )[days - 1]

    def get_last_business_day(self, date: datetime.date) -> datetime.date:
        """
        指定された月の最終営業日を取得

        Args:
            date (datetime.date): 日付

        Returns:
            datetime.date: 最終営業日
        """
        return self.get_business_date_range(
            datetime.date(date.year, date.month, 1),
            self.get_last_day(datetime.date(date.year, date.month, 1)),
        )[-1]

    def print_calendar(self, date: datetime.date):
        """
        カレンダーを出力

        Args:
            date (datetime.date): 対象年月

        """
        from flaretool.constants import ConsoleColor as Color
        year = date.year
        month = date.month

        def format_day(year, month, day, weekday):
            if weekday == 0:
                return f'{Color.RED}{day:2d}{Color.RESET} '
            elif weekday == 6:
                return f'{Color.BLUE}{day:2d}{Color.RESET} '
            else:
                if self.get_holiday_name(datetime.date(year, month, day)):
                    return f'{Color.GREEN}{day:2d}{Color.RESET} '
                else:
                    return f"{day:2d} "
        calendar.setfirstweekday(calendar.SUNDAY)

        # カレンダーを取得
        cal = calendar.monthcalendar(year, month)

        # 月と年を出力
        print("    ", calendar.month_name[month], year)
        print("Su Mo Tu We Th Fr Sa")

        # カレンダーを出力
        for week in cal:
            line = ""
            for w, day in enumerate(week):
                if day == 0:
                    line += "   "
                else:
                    line += format_day(year, month, day, w)
            print(line)
        calendar.setfirstweekday(calendar.MONDAY)

    def get_date_information(self, date: datetime.date) -> tuple:
        """
        指定された日付から週番号、曜日、および祝日の名称を取得

        Args:
            date (datetime.date): 日付

        Returns:
            tuple: 週番号(int)(0-4)、曜日(str)、曜日(int)(0-6)、祝日の名称(str)のタプル
        """
        _, last_day = calendar.monthrange(date.year, date.month)
        first_day = datetime.date(date.year, date.month, 1)
        if first_day.weekday() == 6:
            first_sunday = first_day
        else:
            first_sunday = first_day + \
                datetime.timedelta(days=(6 - first_day.weekday() + 1))
        weeks = ((date - first_sunday).days + 1) // 7
        if date.day > last_day - 6:
            weeks += 1
        week_number = weeks

        weekdays = ['Monday', 'Tuesday', 'Wednesday',
                    'Thursday', 'Friday', 'Saturday', 'Sunday']
        weekday = weekdays[date.weekday()]

        holiday_name = self.get_holiday_name(date)

        return week_number, weekday, date.weekday(), holiday_name



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.holiday.JapaneseHolidays import JapaneseHolidays
from flaretool.holiday.JapaneseHolidaysOnline import JapaneseHolidaysOnline

__all__ = [
    "JapaneseHolidays",
    "JapaneseHolidaysOnline",
]



#!/bin/python
# -*- coding: utf-8 -*-
import os
import tempfile
import socket
import whois
import ipaddress

from flaretool.constants import ADHOST_DATA_URL, BASE_API_URL, COUNTRY_IPV4_DATA_URL, Country
from flaretool.nettool.models import IpInfo, PunyDomainInfo
from flaretool.common import requests
from urllib import robotparser
from urllib.parse import urlparse, urlunparse

from flaretool.logger import get_logger

logger = get_logger()


def get_global_ipaddr_info(addr: str = None) -> IpInfo:
    """
    指定されたグローバルIPアドレスの情報を取得する関数
    (指定がない場合は実行端末のグローバルIPを取得)

    Args:
        addr (str): IPアドレス または ホスト名（デフォルトはNone）

    Returns:
        IpInfo: IPアドレスの情報(取得できない項目はNoneをセット)

    """
    addr = "" if addr is None else f"/{addr}"
    result = requests.get(f"{BASE_API_URL}/ip{addr}").json()
    return IpInfo(**result)


def lookup_ip(domain: str) -> str:
    """
    指定されたドメイン名からIPアドレスを取得する関数

    Args:
        domain (str): ドメイン名

    Returns:
        str: IPアドレス

    """
    try:
        return socket.gethostbyname(domain)
    except socket.gaierror as e:
        logger.error(f"Error during forward DNS lookup: {e}")
        return None


def lookup_domain(ip: str) -> str:
    """
    指定されたIPアドレスからドメイン名を取得する関数

    Args:
        ip (str): IPアドレス

    Returns:
        str: ドメイン名

    """
    try:
        return socket.gethostbyaddr(ip)[0]
    except socket.herror as e:
        logger.error(f"Error during reverse DNS lookup: {e}")
        return None


def is_ip_in_allowed_networks(ipaddr: str, allow_networks: list[str]) -> bool:
    """
    指定されたIPアドレスが指定されたネットワークに属しているかどうかを判定する関数

    Args:
        ipaddr (str): IPアドレス
        allow_networks (list[str]): 許可されたネットワークのリスト

    Returns:
        bool: 指定されたIPアドレスが指定されたネットワークに属している場合はTrue、それ以外の場合はFalse

    """
    ip_networks = [ipaddress.ip_network(network) for network in allow_networks]
    remote_addr = ipaddress.ip_address(ipaddr)
    return any(remote_addr in ip_network for ip_network in ip_networks)


def domain_exists(domain: str) -> bool:
    """
    指定されたドメイン名が存在するかどうかを判定する関数

    Args:
        domain (str): ドメイン名

    Returns:
        bool: 指定されたドメイン名が存在する場合はTrue、それ以外の場合はFalse

    """
    try:
        w = whois.whois(domain)
        return True if w.status else False
    except whois.parser.PywhoisError:
        return False


def get_country_ip_list(country: Country = Country.JP) -> list[str]:
    """
    特定の国のIPアドレスを取得（デフォルトは日本）

    Args:
        country (Country, optional): 国コード. Defaults to Country.JP.

    Returns:
        list[str]: IPアドレスのリスト
    """
    return requests.get(COUNTRY_IPV4_DATA_URL.format(cc=country.name)).text.splitlines()


def is_country_ip(ipaddr: str, country: Country = Country.JP) -> bool:
    """
    指定されたアドレスが指定の国のIPアドレスか確認する関数（デフォルトは日本）

    Args:
        ipaddr (str): IPアドレス
        country (Country, optional): 国コード. Defaults to Country.JP.

    Returns:
        bool: 指定されたIPアドレスが指定した国のIPの場合はTrue、それ以外の場合はFalse
    """
    return is_ip_in_allowed_networks(ipaddr, get_country_ip_list(country))


def get_japanip_list() -> list[str]:
    """
    日本のIPアドレスを取得する関数

    Returns:
        list[str]: 日本のIPアドレスのリスト
    """
    return get_country_ip_list()


def is_japan_ip(ipaddr: str) -> bool:
    """
    指定されたアドレスが日本のIPアドレスか確認する関数

    Args:
        ipaddr (str): IPアドレス

    Returns:
        bool: 指定されたIPアドレスが日本のIPの場合はTrue、それ以外の場合はFalse
    """
    return is_ip_in_allowed_networks(ipaddr, get_japanip_list())


def get_puny_code(domain: str) -> PunyDomainInfo:
    """
    日本語を含むドメインをpunycodeに変換する関数

    Args:
        domain (str): ドメイン名

    Returns:
        PunyDomainInfo: 取得結果
    """
    result = requests.get(f"{BASE_API_URL}/puny/{domain}").json()
    return PunyDomainInfo(**result)


def get_adhost(domain: str = None) -> list[str]:
    """
    広告および危険なホストのリストを取得

    Args:
        domain (str, optional): 検索ドメイン（デフォルトはNone）

    Returns:
        list[str]: ホストリスト
    """

    with tempfile.TemporaryDirectory() as temp_dir:
        file_path = os.path.join(temp_dir, os.path.basename(ADHOST_DATA_URL))

        with open(file_path, "wb") as file:
            response = requests.get(ADHOST_DATA_URL)
            file.write(response.content)

        with open(file_path, "r") as file:
            file_content_list = [
                host for host in file.read().splitlines() if "#" not in host]
            return file_content_list if domain is None else [host for host in file_content_list if domain in host]


def get_robots_txt_url(url: str) -> str:
    """
    スクレイピング対象URLからrobots.txtファイルのURLを生成

    Args:
        url (str): スクレイピング対象のURL

    Returns:
        str: robots.txtファイルのURL
    """
    parsed_url = urlparse(url)
    robots_txt_url = urlunparse(
        (parsed_url.scheme, parsed_url.netloc, '/robots.txt', '', '', ''))
    return robots_txt_url


def is_scraping_allowed(url: str, user_agent: str = "*") -> bool:
    """
    指定されたURLに対してスクレイピングが許可されているかどうかを判定

    Args:
        url (str): スクレイピング対象のURL
        user_agent (str, optional): 使用するユーザーエージェント デフォルトは"*"

    Returns:
        bool: スクレイピングが許可されている場合はTrue、禁止されている場合はFalse
    """
    rp = robotparser.RobotFileParser()
    # robots.txtファイルのURLを設定
    rp.set_url(get_robots_txt_url(url))
    rp.read()
    # 指定されたURLに対するスクレイピングが許可されているかどうかを判断
    return rp.can_fetch(user_agent, url)


__all__ = []



#!/bin/python
# -*- coding: utf-8 -*-
import time
import socket
import inspect
from functools import wraps
from flaretool.errors import FlareToolNetworkError
from flaretool.logger import get_logger

__all__ = [
    "network_required",
    "retry",
    "repeat",
    "type_check",
]


def type_check(func):
    """
    型チェックを行うデコレーター

    Parameters:
        func (callable): 型チェックを行う関数。

    Returns:
        callable: デコレートされた関数。

    Raises:
        TypeError: 引数の型が予期された型と一致しない場合に発生します。

    Examples:
        >>> @type_check
        ... def my_function(arg1: int, arg2: str):
        ...     # 関数の処理...
        ...     pass

        >>> my_function(10, "Hello")
        10 Hello

        >>> my_function("10", 20)
        TypeError: Argument 'arg1' has unexpected type 'str' (expected 'int').
    """
    # デコレートされた関数の引数と型アノテーションを取得
    sig = inspect.signature(func)
    parameters = sig.parameters

    @wraps(func)
    def wrapper(*args, **kwargs):
        # 引数の型チェックを実施
        for name, value in zip(parameters.keys(), args):
            if name in func.__annotations__:
                expected_type = func.__annotations__[name]
                if not isinstance(value, expected_type):
                    raise TypeError(
                        f"Argument '{name}' has unexpected type '{type(value).__name__}' (expected '{expected_type.__name__}')."
                    )

        # キーワード引数がある場合の型チェックを実施
        for name, value in kwargs.items():
            if name in func.__annotations__:
                expected_type = func.__annotations__[name]
                if not isinstance(value, expected_type):
                    raise TypeError(
                        f"Argument '{name}' has unexpected type '{type(value).__name__}' (expected '{expected_type.__name__}')."
                    )

        return func(*args, **kwargs)

    return wrapper


def network_required(func):
    """
    ネットワーク接続のチェックを行い、接続されている場合には指定されたメソッドを実行します。
    接続されていない場合にはFlareToolNetworkErrorを発生させます。

    Args:
        func (callable): ネットワーク接続を確認した後に実行するメソッド。

    Returns:
        callable: デコレートされた関数。

    Raises:
        FlareToolNetworkError: ネットワークに接続されていない場合に発生する例外。
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            socket.create_connection(("google.com", 443), timeout=5)
        except OSError:
            raise FlareToolNetworkError("ネットワークに接続されていません。")
        return func(*args, **kwargs)

    return wrapper


def retry(max_attempts: int, delay: int = 1):
    """
    リトライ処理を行うデコレーター。

    Args:
        max_attempts (int): 最大リトライ回数
        delay (int, optional): リトライ間隔（秒）（デフォルトは1秒）

    Returns:
        function: デコレートされた関数を実行する

    Examples:
        @retry(max_attempts=3, delay=2)
        def example_function():
            pass
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            """
            リトライを行うラッパー関数

            Args:
                *args: 関数への位置引数
                **kwargs: 関数へのキーワード引数

            Returns:
                Any: 関数の戻り値（もしくはリトライ上限に達した場合はNone）
            """
            logger = get_logger()
            attempts = 0
            while attempts < max_attempts:
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    attempts += 1
                    if attempts >= max_attempts:
                        logger.error(f"Max attempts reached. Giving up.")
                        raise e
                    logger.error(
                        f"Error occurred: {e}. Retrying in {delay} second(s).")
                    time.sleep(delay)
        return wrapper
    return decorator


def repeat(repeat_count: int, interval: int = 0):
    """
    指定された間隔で関数を指定回数再実行するデコレーター。

    Args:
        repeat_count (int): リピートする回数
        interval (int, optional): 実行間隔（秒単位）。デフォルト値は0。

    Returns:
        function: デコレートされた関数

    Examples:
        >>> @repeat(5, 3)
        ... def print_hello():
        ...     print("Hello, world!")
        ...     if some_condition: # 実行を止めたい場合の条件
        ...         raise StopIteration("Stop the loop")
        ...
        >>> print_hello()
        Hello, world!
        Hello, world!
        Hello, world!
        Hello, world!
        Hello, world!
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            logger = get_logger()
            for _ in range(repeat_count):
                try:
                    result = func(*args, **kwargs)
                except StopIteration as e:
                    logger.error(str(e))
                    break
                time.sleep(interval)
            return result
        return wrapper
    return decorator



#!/bin/python
# -*- coding: utf-8 -*-
import base64
import hashlib
import os
import tempfile
import time
import flaretool
from flaretool.enums import ConversionMode, Base64Mode, HashMode


ASCII_ZENKAKU_CHARS = (
    u'ａ', u'ｂ', u'ｃ', u'ｄ', u'ｅ', u'ｆ', u'ｇ', u'ｈ', u'ｉ', u'ｊ', u'ｋ',
    u'ｌ', u'ｍ', u'ｎ', u'ｏ', u'ｐ', u'ｑ', u'ｒ', u'ｓ', u'ｔ', u'ｕ', u'ｖ',
    u'ｗ', u'ｘ', u'ｙ', u'ｚ',
    u'Ａ', u'Ｂ', u'Ｃ', u'Ｄ', u'Ｅ', u'Ｆ', u'Ｇ', u'Ｈ', u'Ｉ', u'Ｊ', u'Ｋ',
    u'Ｌ', u'Ｍ', u'Ｎ', u'Ｏ', u'Ｐ', u'Ｑ', u'Ｒ', u'Ｓ', u'Ｔ', u'Ｕ', u'Ｖ',
    u'Ｗ', u'Ｘ', u'Ｙ', u'Ｚ',
    u'！', u'”', u'＃', u'＄', u'％', u'＆', u'’', u'（', u'）', u'＊', u'＋',
    u'，', u'－', u'．', u'／', u'：', u'；', u'＜', u'＝', u'＞', u'？', u'＠',
    u'［', u'￥', u'］', u'＾', u'＿', u'‘', u'｛', u'｜', u'｝', u'～', u'　'
)

ASCII_HANKAKU_CHARS = (
    u'a', u'b', u'c', u'd', u'e', u'f', u'g', u'h', u'i', u'j', u'k',
    u'l', u'm', u'n', u'o', u'p', u'q', u'r', u's', u't', u'u', u'v',
    u'w', u'x', u'y', u'z',
    u'A', u'B', u'C', u'D', u'E', u'F', u'G', u'H', u'I', u'J', u'K',
    u'L', u'M', u'N', u'O', u'P', u'Q', u'R', u'S', u'T', u'U', u'V',
    u'W', u'X', u'Y', u'Z',
    u'!', u'"', u'#', u'$', u'%', u'&', u'\'', u'(', u')', u'*', u'+',
    u',', u'-', u'.', u'/', u':', u';', u'<', u'=', u'>', u'?', u'@',
    u'[', u"¥", u']', u'^', u'_', u'`', u'{', u'|', u'}', u'~', u' '
)

KANA_ZENKAKU_CHARS = (
    u'ア', u'イ', u'ウ', u'エ', u'オ', u'カ', u'キ', u'ク', u'ケ', u'コ',
    u'サ', u'シ', u'ス', u'セ', u'ソ', u'タ', u'チ', u'ツ', u'テ', u'ト',
    u'ナ', u'ニ', u'ヌ', u'ネ', u'ノ', u'ハ', u'ヒ', u'フ', u'ヘ', u'ホ',
    u'マ', u'ミ', u'ム', u'メ', u'モ', u'ヤ', u'ユ', u'ヨ',
    u'ラ', u'リ', u'ル', u'レ', u'ロ', u'ワ', u'ヲ', u'ン',
    u'ァ', u'ィ', u'ゥ', u'ェ', u'ォ', u'ッ', u'ャ', u'ュ', u'ョ',
    u'。', u'、', u'・', u'゛', u'゜', u'「', u'」', u'ー'
)

KANA_HANKAKU_CHARS = (
    u'ｱ', u'ｲ', u'ｳ', u'ｴ', u'ｵ', u'ｶ', u'ｷ', u'ｸ', u'ｹ', u'ｺ',
    u'ｻ', u'ｼ', u'ｽ', u'ｾ', u'ｿ', u'ﾀ', u'ﾁ', u'ﾂ', u'ﾃ', u'ﾄ',
    u'ﾅ', u'ﾆ', u'ﾇ', u'ﾈ', u'ﾉ', u'ﾊ', u'ﾋ', u'ﾌ', u'ﾍ', u'ﾎ',
    u'ﾏ', u'ﾐ', u'ﾑ', u'ﾒ', u'ﾓ', u'ﾔ', u'ﾕ', u'ﾖ',
    u'ﾗ', u'ﾘ', u'ﾙ', u'ﾚ', u'ﾛ', u'ﾜ', u'ｦ', u'ﾝ',
    u'ｧ', u'ｨ', u'ｩ', u'ｪ', u'ｫ', u'ｯ', u'ｬ', u'ｭ', u'ｮ',
    u'｡', u'､', u'･', u'ﾞ', u'ﾟ', u'｢', u'｣', u'ｰ'
)

DIGIT_ZENKAKU_CHARS = (
    u'０', u'１', u'２', u'３', u'４', u'５', u'６', u'７', u'８', u'９'
)

DIGIT_HANKAKU_CHARS = (
    u'0', u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9'
)

KANA_TEN_MAP = (
    (u'ガ', u'ｶ'), (u'ギ', u'ｷ'), (u'グ', u'ｸ'), (u'ゲ', u'ｹ'), (u'ゴ', u'ｺ'),
    (u'ザ', u'ｻ'), (u'ジ', u'ｼ'), (u'ズ', u'ｽ'), (u'ゼ', u'ｾ'), (u'ゾ', u'ｿ'),
    (u'ダ', u'ﾀ'), (u'ヂ', u'ﾁ'), (u'ヅ', u'ﾂ'), (u'デ', u'ﾃ'), (u'ド', u'ﾄ'),
    (u'バ', u'ﾊ'), (u'ビ', u'ﾋ'), (u'ブ', u'ﾌ'), (u'ベ', u'ﾍ'), (u'ボ', u'ﾎ'),
    (u'ヴ', u'ｳ')
)

KANA_MARU_MAP = (
    (u'パ', u'ﾊ'), (u'ピ', u'ﾋ'), (u'プ', u'ﾌ'), (u'ペ', u'ﾍ'), (u'ポ', u'ﾎ')
)


def convert_value(value: str, mode: ConversionMode = ConversionMode.HALF_WIDTH, ascii=True, digit=True, kana=True, non_convert_chars=None) -> str:
    """
    文字列を変換(半角/全角/大文字/小文字)

    Args:
        value (str): 変換する文字列
        mode (ConversionMode, optional): 変換モード. デフォルトはConversionMode.HALF_WIDTH.
        ascii (bool, optional): ASCII文字の変換を有効にするかどうか. デフォルトはTrue.
        digit (bool, optional): 数字の変換を有効にするかどうか. デフォルトはTrue.
        kana (bool, optional): カナ文字の変換を有効にするかどうか. デフォルトはTrue.
        non_convert_chars (str or list[str], optional): 変換しない文字列. デフォルトはNone.

    Returns:
        str: 変換後の文字列

    Raises:
        ValueError: 無効な変換モードが指定された場合
    """

    if non_convert_chars is None:
        non_convert_chars = []
    elif isinstance(non_convert_chars, str):
        non_convert_chars = [non_convert_chars]

    if mode in [ConversionMode.UPPER, ConversionMode.LOWER]:
        temp = []
        for c in value:
            if c in non_convert_chars:
                temp.append(c)
            else:
                temp.append(c.upper() if mode ==
                            ConversionMode.UPPER else c.lower())
        return ''.join(temp)

    tables = {
        'ascii_zh_table': {},
        'ascii_hz_table': {},
        'kana_zh_table': {},
        'kana_hz_table': {},
        'digit_zh_table': {},
        'digit_hz_table': {},
        'kana_ten_zh_table': {},
        'kana_ten_hz_table': {},
        'kana_maru_zh_table': {},
        'kana_maru_hz_table': {},
    }

    for (z, h) in zip(ASCII_ZENKAKU_CHARS, ASCII_HANKAKU_CHARS):
        tables['ascii_zh_table'][z] = h
        tables['ascii_hz_table'][h] = z

    for (z, h) in zip(KANA_ZENKAKU_CHARS, KANA_HANKAKU_CHARS):
        tables['kana_zh_table'][z] = h
        tables['kana_hz_table'][h] = z

    for (z, h) in zip(DIGIT_ZENKAKU_CHARS, DIGIT_HANKAKU_CHARS):
        tables['digit_zh_table'][z] = h
        tables['digit_hz_table'][h] = z

    for (z, h) in KANA_TEN_MAP:
        tables['kana_ten_zh_table'][z] = h
        tables['kana_ten_hz_table'][h] = z

    for (z, h) in KANA_MARU_MAP:
        tables['kana_maru_zh_table'][z] = h
        tables['kana_maru_hz_table'][h] = z

    temp = []
    prev = ''
    for c in value:
        if c in non_convert_chars:
            temp.append(c)
            prev = c
            continue

        if mode == ConversionMode.HALF_WIDTH:
            if ascii and c in tables['ascii_zh_table']:
                temp.append(tables['ascii_zh_table'][c])
            elif digit and c in tables['digit_zh_table']:
                temp.append(tables['digit_zh_table'][c])
            elif kana and c in tables['kana_zh_table']:
                temp.append(tables['kana_zh_table'][c])
            elif kana and c in tables['kana_ten_zh_table']:
                temp.append(tables['kana_ten_zh_table'][c])
                temp.append('ﾞ')
            elif kana and c in tables['kana_maru_zh_table']:
                temp.append(tables['kana_maru_zh_table'][c])
                temp.append('ﾟ')
            else:
                temp.append(c)
        elif mode == ConversionMode.FULL_WIDTH:
            if ascii and c in tables['ascii_hz_table']:
                temp.append(tables['ascii_hz_table'][c])
            elif digit and c in tables['digit_hz_table']:
                temp.append(tables['digit_hz_table'][c])
            elif kana and c == 'ﾞ' and prev in tables['kana_ten_hz_table']:
                temp.pop()
                temp.append(tables['kana_ten_hz_table'][prev])
            elif kana and c == 'ﾟ' and prev in tables['kana_maru_hz_table']:
                temp.pop()
                temp.append(tables['kana_maru_hz_table'][prev])
            elif kana and c in tables['kana_hz_table']:
                temp.append(tables['kana_hz_table'][c])
            else:
                temp.append(c)
        else:
            raise ValueError("Invalid conversion mode")
        prev = c
    return ''.join(temp)


def base64_convert(value: str, mode: Base64Mode = Base64Mode.ENCODE, encoding: str = 'utf-8') -> str:
    """
    文字列のBase64エンコードまたはデコードを行います。

    Args:
        value (str): エンコードまたはデコードする文字列
        mode (Base64Mode, optional): 処理モード. デフォルトはBase64Mode.ENCODE.
        encoding (str, optional): エンコードに使用する文字エンコーディング. デフォルトは'utf-8'.

    Returns:
        str: エンコードまたはデコード後の文字列

    Raises:
        ValueError: 無効な処理モードが指定された場合
    """
    if mode == Base64Mode.ENCODE:
        encoded_bytes = value.encode(encoding)
        encoded_value = base64.b64encode(encoded_bytes).decode(encoding)
        return encoded_value
    elif mode == Base64Mode.DECODE:
        decoded_bytes = base64.b64decode(value.encode(encoding))
        decoded_value = decoded_bytes.decode(encoding)
        return decoded_value
    else:
        raise ValueError("Invalid base64 mode")


def hash_value(value: str, mode: HashMode = HashMode.MD5, encoding: str = 'utf-8') -> str:
    """
    文字列のハッシュ計算を行います。

    Args:
        value (str): ハッシュ計算する文字列
        mode (HashMode, optional): ハッシュモード. デフォルトはHashMode.MD5.
        encoding (str, optional): 文字列のエンコーディング. デフォルトは'utf-8'.

    Returns:
        str: ハッシュ値の16進数表現

    Raises:
        ValueError: 無効なハッシュモードが指定された場合
    """
    if mode == HashMode.MD5:
        hash_object = hashlib.md5()
    elif mode == HashMode.SHA1:
        hash_object = hashlib.sha1()
    elif mode == HashMode.SHA256:
        hash_object = hashlib.sha256()
    elif mode == HashMode.SHA512:
        hash_object = hashlib.sha512()
    else:
        raise ValueError("Invalid hash mode")

    hash_object.update(value.encode(encoding))
    return hash_object.hexdigest()


class DictToFieldConverter:
    """
    This class converts a dictionary into field values.

    Args:
        dictionary (dict): The dictionary to convert.

    Attributes:
        dictionary (dict): The dictionary being converted.

    Raises:
        AttributeError: If a requested field does not exist in the dictionary.

    Example:
        # Create a dictionary
        dictionary = {'name': 'John', 'age': 30, 'city': 'Tokyo'}

        # Create an instance of DictToFieldConverter
        converter = DictToFieldConverter(dictionary)

        # Access field values
        name = converter.name
        age = converter.age
        city = converter.city

        print(name)  # Output: 'John'
        print(age)   # Output: 30
        print(city)  # Output: 'Tokyo'
    """

    def __init__(self, dictionary):
        self.dictionary = dictionary

    def __getattr__(self, field_name):
        if field_name in self.dictionary:
            return self.dictionary[field_name]
        else:
            raise AttributeError(
                f"'DictToFieldConverter' object has no attribute '{field_name}'")


def get_temp_dir_path():
    """
    Get the path to a temporary dir path.

    Returns:
        str: The path to the temporary path.
    """
    temp_dir = os.path.join(
        tempfile.gettempdir(),
        "python_{}".format(flaretool.__name__),
    )
    os.makedirs(temp_dir, exist_ok=True)
    return temp_dir


def is_file_fresh(file_path: str, days: int) -> bool:
    """
    Check if the given file has been modified within the specified number of days.

    Args:
        file_path (str): The path to the file.
        days (int): The number of days to check for freshness.

    Returns:
        bool: True if the file is fresh, False otherwise.
    """
    try:
        file_modified_time = os.path.getmtime(file_path)
        current_time = time.time()
        time_difference = current_time - file_modified_time
        return time_difference <= (days * 24 * 60 * 60)
    except FileNotFoundError:
        return False



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.errors import FlareToolError


class ShortUrlError(FlareToolError):

    def __init__(self, message, author=None, response=None, data=None):
        self.message = message
        self.author = author
        self.response = response
        self.data = data
        super().__init__(self.message)


class ShortUrlAuthenticationError(ShortUrlError):
    pass


class ShortUrlDataUpdateError(ShortUrlError):
    pass



#!/bin/python
# -*- coding: utf-8 -*-
from typing import Union
from flaretool.basemodels import BaseDataModel


class DdnsInfo(BaseDataModel):
    """
    Represents the Dynamic DNS (DDNS) information.

    Attributes:
        result (int): The result code of the DDNS update.
        status (str): The status of the DDNS update.
        currentIp (str): The current IP address associated with the domain.
        updateIp (str): The IP address that was updated.
        domain (str): The domain associated with the DDNS update.
    """
    result: int
    status: str
    currentIp: Union[str, None] = None
    updateIp: str
    domain: str



#!/bin/python
# -*- coding: utf-8 -*-
import sys
import datetime
from typing import Any, Union
import warnings
from flaretool.holiday import JapaneseHolidays
from flaretool.holiday.errors import JapaneseHolidaysError
from flaretool.holiday.models import HolidaysResponseModel
from flaretool.common import requests
from flaretool.decorators import network_required
from flaretool.logger import get_logger
from flaretool.constants import BASE_API_URL

logger = get_logger()


class JapaneseHolidaysOnline(JapaneseHolidays):
    """
    日本の祝日を管理するクラス(オンライン版)
    """

    def __init__(self):
        super().__init__()
        self.holidays = self.__request_get_holiday()
        self.version = self.holidays.version
        self.__is_warning = False
        logger.info(str(self).replace("\n", " "))

    def __str__(self):
        return "{}\n{} v{}\nPowered By {}\nUpdated {}\nSupported {} ~ {}\n".format(
            self.__class__.__name__,
            self.holidays.title,
            self.holidays.version,
            self.holidays.author,
            self.holidays.updated,
            self.holidays.supported.from_date,
            self.holidays.supported.to_date,
        )

    def show_info(self):
        """
        祝日の対応情報を表示
        """
        print(self.__str__())

    @network_required
    def __request_get_holiday(self) -> HolidaysResponseModel:
        response = requests.get(f"{BASE_API_URL}/japanholiday.json")
        result = response.json()
        holidays = HolidaysResponseModel(**result)
        if not holidays.status:
            warnings.warn(holidays.message, Warning)
        if holidays.count == 0:
            raise JapaneseHolidaysError(message=holidays.message)
        return holidays

    def __super_method(self, method: str, date: datetime.date):
        if not self.holidays.supported.from_date <= date <= self.holidays.supported.to_date:
            if not self.__is_warning:
                message = "The online version does not support '{}'. It supports dates from '{}' to '{}'. Therefore, I will execute it offline.".format(
                    date, self.holidays.supported.from_date, self.holidays.supported.to_date)
                warnings.warn(message, Warning)
                self.__is_warning = True
            return True, getattr(super(), method)(date)
        return False, None

    def get_holiday_name(self, date: Union[str, datetime.datetime, datetime.date]) -> Union[str, None]:
        date = self.to_date(date)

        unsupport, is_holiday = self.__super_method(
            sys._getframe().f_code.co_name, date)
        if unsupport:
            return is_holiday

        if self.is_additional_holiday(date):
            return self._additional_holidays.get(date)
        return self.holidays.holidays.get(date, None)

    def __is_holiday_not_substitute(self, method, holiday_name, date) -> bool:
        unsupport, is_holiday = self.__super_method(method, date)
        return is_holiday if unsupport else \
            date in [date for date, name in self.holidays.holidays.items()
                     if holiday_name in name and not "振替" in name]

    def is_new_year(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "元日", date)

    def is_coming_of_age_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "成人の日", date)

    def is_foundation_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "建国記念", date)

    def is_spring_equinox(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "春分の日", date)

    def is_showa_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "昭和の日", date)

    def is_constitution_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "憲法記念日", date)

    def is_greenery_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "みどりの日", date)

    def is_childrens_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "こどもの日", date)

    def is_marine_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "海の日", date)

    def is_mountain_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "山の日", date)

    def is_respect_for_the_aged_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "敬老の日", date)

    def is_autumn_equinox(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "秋分の日", date)

    def is_health_and_sports_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "体育の日", date) \
            or self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "スポーツの日", date)

    def is_culture_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "文化の日", date)

    def is_labour_thanksgiving_day(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "勤労感謝", date)

    def is_emperors_birthday(self, date: datetime.date) -> bool:
        return self.__is_holiday_not_substitute(sys._getframe().f_code.co_name, "天皇誕生日", date)



#!/bin/python
# -*- coding: utf-8 -*-
from flaretool.errors import FlareToolError


class DdnsError(FlareToolError):

    def __init__(self, result, status, currentIp, updateIp, domain):
        self.result = result
        self.status = status
        self.currentIp = currentIp
        self.updateIp = updateIp
        self.domain = domain
        super().__init__(self.status)


class DdnsAuthenticationError(DdnsError):
    pass



#!/bin/python
# -*- coding: utf-8 -*-
import os
from typing import Union
# try:
from pydantic_settings import BaseSettings as bs
# except ImportError:
#     from pydantic import BaseSettings as bs

from dotenv import load_dotenv


class BaseSettings(bs):
    api_key: Union[str, None] = None

    class Config:
        env_prefix = ""

        @classmethod
        def _load_env_file(cls):
            load_dotenv()

        @classmethod
        def _get_environment_variable(cls, name, default):
            return os.getenv(name, default)

    @classmethod
    def get(cls):
        return cls()


def get_settings():
    return BaseSettings()
